@@ -23,40 +23,32 @@
  * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  *						      *
- * Copyright 2002 Sun Microsystems, Inc.  All rights reserved.
+ * Copyright (c) 2002, Oracle and/or its affiliates. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the
- * "Software"), to deal in the Software without restriction, including
- * without limitation the rights to use, copy, modify, merge, publish,
- * distribute, and/or sell copies of the Software, and to permit persons
- * to whom the Software is furnished to do so, provided that the above
- * copyright notice(s) and this permission notice appear in all copies of
- * the Software and that both the above copyright notice(s) and this
- * permission notice appear in supporting documentation.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
- * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
- * OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
- * HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
- * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- * 
- * Except as contained in this notice, the name of a copyright holder
- * shall not be used in advertising or otherwise to promote the sale, use
- * or other dealings in this Software without prior written authorization
- * of the copyright holder.
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * ************************************************** */
 
 #include "scope.h"
 #include "x11.h"
 
-static void DumpHexBuffer(unsigned char *buf, long n);
-
 /*
   For each of the types we need a way to print that type.
   Types are of varieties:
@@ -76,8 +68,8 @@ static void DumpHexBuffer(unsigned char *buf, long n);
 
 
 /* print representation of a character for debugging */
-static char   *printrep (c)
-     unsigned short  c;
+static const char *
+printrep (unsigned short  c)
 {
   static char pr[8];
 
@@ -132,12 +124,12 @@ static char   *printrep (c)
 */
 
 #define MaxIndent 10
-static char Leader[MaxIndent + 1];
+char Leader[MaxIndent + 1];
 static short    CurrentLevel = 0;
 
 void
-SetIndentLevel(which)
-     short   which;
+SetIndentLevel (
+    short   which)
 {
   short   i;
 
@@ -157,13 +149,14 @@ SetIndentLevel(which)
 }
 
 static void
-ModifyIndentLevel(amount)
-     short   amount;
+ModifyIndentLevel (
+     short   amount)
 {
   SetIndentLevel(CurrentLevel + amount);
 }
 
-static short   SizeofLeader ()
+static short
+SizeofLeader (void)
 {
   return (CurrentLevel * 8);
 }
@@ -176,11 +169,11 @@ static short   SizeofLeader ()
 /* if we want verbose enough output, we will dump the buffer in hex */
 
 void
-DumpItem(name, fd, buf, n)
-     char   *name;
-     FD      fd;
-     unsigned char *buf;
-     long    n;
+DumpItem (
+    const char   *name,
+    FD      fd,
+    const unsigned char *buf,
+    long    n)
 {
   if (n == 0)
     return;
@@ -197,64 +190,69 @@ DumpItem(name, fd, buf, n)
 /* ************************************************************ */
 
 int
-PrintINT8(unsigned char *buf)
+PrintINT8(const unsigned char *buf)
 {
   /* print a INT8 -- 8-bit signed integer */
   short   n = IByte (buf);
   if (n > 127)
     n = 256 - n;
-  return fprintf(stdout, "%d", n);
+  fprintf(stdout, "%d", n);
+  return 1;
 }
 
 int
-PrintINT16(unsigned char *buf)
+PrintINT16(const unsigned char *buf)
 {
   /* print a INT16 -- 16-bit signed integer */
   long    n = IShort (buf);
   if (n > 32767)
     n = 65536 - n;
-  return fprintf(stdout, "%d", n);
+  fprintf(stdout, "%ld", n);
+  return 2;
 }
 
 int
-PrintINT32(unsigned char *buf)
+PrintINT32(const unsigned char *buf)
 {
   /* print a INT32 -- 32-bit signed integer */
   long    n = ILong (buf);
-  return fprintf(stdout, "%d", n);
+  fprintf(stdout, "%ld", n);
+  return 4;
 }
 
 /* ************************************************************ */
 
 int
-PrintCARD8(unsigned char *buf)
+PrintCARD8(const unsigned char *buf)
 {
   /* print a CARD8 -- 8-bit unsigned integer */
   unsigned short   n = IByte (buf);
-  return fprintf(stdout, "%02x", (unsigned)(n & 0xff));
+  fprintf(stdout, "%02x", (unsigned)(n & 0xff));
+  return 1;
 }
 
 int
-PrintCARD16(unsigned char *buf)
+PrintCARD16(const unsigned char *buf)
 {
   /* print a CARD16 -- 16-bit unsigned integer */
   unsigned long   n = IShort (buf);
-  return fprintf(stdout, "%04x", (unsigned)(n & 0xffff));
+  fprintf(stdout, "%04x", (unsigned)(n & 0xffff));
+  return 1;
 }
 
 int
-PrintCARD32(unsigned char *buf)
+PrintCARD32(const unsigned char *buf)
 {
   /* print a CARD32 -- 32-bit unsigned integer */
   unsigned long   n = ILong (buf);
-  fprintf(stdout, "%08x", n);
+  fprintf(stdout, "%08lx", n);
   return(4);
 }
 
 /* ************************************************************ */
 
 int
-PrintBYTE(unsigned char *buf)
+PrintBYTE(const unsigned char *buf)
 {
   /* print a BYTE -- 8-bit value */
   short   n = IByte (buf);
@@ -264,7 +262,7 @@ PrintBYTE(unsigned char *buf)
 
 
 int
-PrintCHAR8(unsigned char *buf)
+PrintCHAR8(const unsigned char *buf)
 {
   /* print a CHAR8 -- 8-bit character */
   unsigned short   n = IByte (buf);
@@ -274,16 +272,16 @@ PrintCHAR8(unsigned char *buf)
 
 
 int
-PrintSTRING16(unsigned char *buf)
+PrintSTRING16(const unsigned char *buf)
 {
-  /* print a CHAR16 -- 16-bit character */
-  unsigned short   n = IShort (buf);
+  /* print a CHAR2B -- 16-bit character which is never byte-swapped */
+  unsigned short   n = IChar2B (buf);
   fprintf(stdout, "%s", printrep(n));
-  return(1);
+  return 2 + n;
 }
 
 int
-PrintSTR(unsigned char *buf)
+PrintSTR(const unsigned char *buf)
 {
   /* STR have the length (1 byte) then a string of CHAR8 */
   short   n;
@@ -298,158 +296,169 @@ PrintSTR(unsigned char *buf)
 /* ************************************************************ */
 
 int
-PrintWINDOW(unsigned char *buf)
+PrintWINDOW(const unsigned char *buf)
 {
   /* print a WINDOW -- CARD32  plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
     fprintf(stdout, "None");
   else
-    fprintf(stdout, "WIN %08x", n);
+    fprintf(stdout, "WIN %08lx", n);
   return(4);
 }
 
 int
-PrintWINDOWD(unsigned char *buf)
+PrintWINDOWD(const unsigned char *buf)
 {
   /* print a WINDOWD -- CARD32  plus 0 = PointerWindow, 1 = InputFocus */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "PointerWindow");
+    fprintf(stdout, "PointerWindow");
   else if (n == 1)
-      return fprintf(stdout, "InputFocus");
+      fprintf(stdout, "InputFocus");
     else
-      return PrintWINDOW(buf);
+      (void)PrintWINDOW(buf);
+  return 4;
 }
 
 int
-PrintWINDOWNR(unsigned char *buf)
+PrintWINDOWNR(const unsigned char *buf)
 {
   /* print a WINDOWNR -- CARD32  plus 0 = None, 1 = PointerRoot */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else if (n == 1)
-      return fprintf(stdout, "PointerRoot");
+      fprintf(stdout, "PointerRoot");
     else
-      return PrintWINDOW(buf);
+      (void)PrintWINDOW(buf);
+  return 4;
 }
 
 
 int
-PrintPIXMAP(unsigned char *buf)
+PrintPIXMAP(const unsigned char *buf)
 {
   /* print a PIXMAP -- CARD32  plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else
-    return fprintf(stdout, "PXM %08x", n);
+    fprintf(stdout, "PXM %08lx", n);
+  return 4;
 }
 
 int
-PrintPIXMAPNPR(unsigned char *buf)
+PrintPIXMAPNPR(const unsigned char *buf)
 {
   /* print a PIXMAPNPR -- CARD32  plus 0 = None, 1 = ParentRelative */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else if (n == 1)
-      return fprintf(stdout, "ParentRelative");
+      fprintf(stdout, "ParentRelative");
     else
-      return PrintPIXMAP(buf);
+      PrintPIXMAP(buf);
+  return 4;
 }
 
 int
-PrintPIXMAPC(unsigned char *buf)
+PrintPIXMAPC(const unsigned char *buf)
 {
   /* print a PIXMAPC -- CARD32  plus 0 = CopyFromParent */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "CopyFromParent");
+    fprintf(stdout, "CopyFromParent");
   else
-    return PrintPIXMAP(buf);
+    PrintPIXMAP(buf);
+  return 4;
 }
 
 
 int
-PrintCURSOR(unsigned char *buf)
+PrintCURSOR(const unsigned char *buf)
 {
   /* print a CURSOR -- CARD32  plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else
-    return fprintf(stdout, "CUR %08x", n);
+    fprintf(stdout, "CUR %08lx", n);
+  return 4;
 }
 
 
 int
-PrintFONT(unsigned char *buf)
+PrintFONT(const unsigned char *buf)
 {
   /* print a FONT -- CARD32  plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else
-    return fprintf(stdout, "FNT %08x", n);
+    fprintf(stdout, "FNT %08lx", n);
+  return 4;
 }
 
 
 int
-PrintGCONTEXT(unsigned char *buf)
+PrintGCONTEXT(const unsigned char *buf)
 {
   /* print a GCONTEXT -- CARD32 */
   long    n = ILong (buf);
-  return fprintf(stdout, "GXC %08x", n);
+  fprintf(stdout, "GXC %08lx", n);
+  return 4;
 }
 
 
 int
-PrintCOLORMAP(unsigned char *buf)
+PrintCOLORMAP(const unsigned char *buf)
 {
   /* print a COLORMAP -- CARD32 plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
     fprintf(stdout, "None");
   else
-    fprintf(stdout, "CMP %08x", n);
+    fprintf(stdout, "CMP %08lx", n);
   return(4);
 }
 
 int
-PrintCOLORMAPC(unsigned char *buf)
+PrintCOLORMAPC(const unsigned char *buf)
 {
   /* print a COLORMAPC -- CARD32 plus 0 = CopyFromParent */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "CopyFromParent");
+    fprintf(stdout, "CopyFromParent");
   else
-    return PrintCOLORMAP(buf);
+    (void)PrintCOLORMAP(buf);
+  return 4;
 }
 
 
 int
-PrintDRAWABLE(unsigned char *buf)
+PrintDRAWABLE(const unsigned char *buf)
 {
   /* print a DRAWABLE -- CARD32 */
   long    n = ILong (buf);
-  return fprintf(stdout, "DWB %08x", n);
+  fprintf(stdout, "DWB %08lx", n);
+  return 4;
 }
 
 int
-PrintFONTABLE(unsigned char *buf)
+PrintFONTABLE(const unsigned char *buf)
 {
   /* print a FONTABLE -- CARD32 */
   long    n = ILong (buf);
-  return fprintf(stdout, "FTB %08x", n);
+  fprintf(stdout, "FTB %08lx", n);
+  return 4;
 }
 
 /* ************************************************************ */
 
 #define NumberofAtoms 68
 
-static char   *AtomTable[NumberofAtoms + 1] =
+static const char   * const AtomTable[NumberofAtoms + 1] =
 {
   "NONE", "PRIMARY", "SECONDARY", "ARC", "ATOM", "BITMAP", "CARDINAL",
   "COLORMAP", "CURSOR", "CUT_BUFFER0", "CUT_BUFFER1", "CUT_BUFFER2",
@@ -472,87 +481,92 @@ static char   *AtomTable[NumberofAtoms + 1] =
    the user defined ones, too. */
 
 int
-PrintATOM(unsigned char *buf)
+PrintATOM(const unsigned char *buf)
 {
   /* print a ATOM -- CARD32 plus 0 = None */
   long    n = ILong (buf);
   if (0 <= n && n <= NumberofAtoms)
     fprintf(stdout, "<%s>", AtomTable[n]);
   else
-    fprintf(stdout, "ATM %08x", n);
+    fprintf(stdout, "ATM %08lx", n);
   return(4);
 }
 
 int
-PrintATOMT(unsigned char *buf)
+PrintATOMT(const unsigned char *buf)
 {
   /* print a ATOMT -- CARD32 plus 0 = AnyPropertyType */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "AnyPropertyType");
+    fprintf(stdout, "AnyPropertyType");
   else
-    return PrintATOM(buf);
+    (void)PrintATOM(buf);
+  return 4;
 }
 
 
 int
-PrintVISUALID(unsigned char *buf)
+PrintVISUALID(const unsigned char *buf)
 {
   /* print a VISUALID -- CARD32 plus 0 = None */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "None");
+    fprintf(stdout, "None");
   else
-    return fprintf(stdout, "VIS %08x", n);
+    fprintf(stdout, "VIS %08lx", n);
+  return 4;
 }
 
 int
-PrintVISUALIDC(unsigned char *buf)
+PrintVISUALIDC(const unsigned char *buf)
 {
   /* print a VISUALIDC -- CARD32 plus 0 = CopyFromParent */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "CopyFromParent");
+    fprintf(stdout, "CopyFromParent");
   else
-    return PrintVISUALID(buf);
+    PrintVISUALID(buf);
+  return 4;
 }
 
 
 int
-PrintTIMESTAMP(unsigned char *buf)
+PrintTIMESTAMP(const unsigned char *buf)
 {
   /* print a TIMESTAMP -- CARD32 plus 0 as the current time */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "CurrentTime");
+    fprintf(stdout, "CurrentTime");
   else
-    return fprintf(stdout, "TIM %08x", n);
+    fprintf(stdout, "TIM %08lx", n);
+  return 4;
 }
 
 
 int
-PrintRESOURCEID(unsigned char *buf)
+PrintRESOURCEID(const unsigned char *buf)
 {
   /* print a RESOURCEID -- CARD32 plus 0 = AllTemporary */
   long    n = ILong (buf);
   if (n == 0)
-    return fprintf(stdout, "AllTemporary");
+    fprintf(stdout, "AllTemporary");
   else
-    return fprintf(stdout, "RID %08x", n);
+    fprintf(stdout, "RID %08lx", n);
+  return 4;
 }
 
 
 int
-PrintKEYSYM(unsigned char *buf)
+PrintKEYSYM(const unsigned char *buf)
 {
   /* print a KEYSYM -- CARD32 */
   long    n = ILong (buf);
-  fprintf(stdout, "KYS %08x", n);
+  fprintf(stdout, "KYS %08lx", n);
   return(4);
 }
 
 int
-PrintKEYCODE(unsigned char *buf)
+PrintKEYCODE(const unsigned char *buf)
 {
   /* print a KEYCODE -- CARD8 */
   unsigned short n = IByte (buf);
@@ -561,51 +575,55 @@ PrintKEYCODE(unsigned char *buf)
 }
 
 int
-PrintKEYCODEA(unsigned char *buf)
+PrintKEYCODEA(const unsigned char *buf)
 {
   /* print a KEYCODEA -- CARD8 plus 0 = AnyKey */
   long    n = IByte (buf);
   if (n == 0)
-    return fprintf(stdout, "AnyKey");
+    fprintf(stdout, "AnyKey");
   else
-    return PrintKEYCODE(buf);
+    (void)PrintKEYCODE(buf);
+  return 1;
 }
 
 
 int
-PrintBUTTON(unsigned char *buf)
+PrintBUTTON(const unsigned char *buf)
 {
   /* print a BUTTON -- CARD8 */
   unsigned short n = IByte (buf);
-  return fprintf(stdout, "%d (%s)", n, printrep(n));
+  fprintf(stdout, "%d (%s)", n, printrep(n));
+  return 1;
 }
 
 int
-PrintBUTTONA(unsigned char *buf)
+PrintBUTTONA(const unsigned char *buf)
 {
   /* print a BUTTONA -- CARD8 plus 0 = AnyButton */
   long    n = IByte (buf);
   if (n == 0)
-    return fprintf(stdout, "AnyButton");
+    fprintf(stdout, "AnyButton");
   else
-    return PrintBUTTON(buf);
+    PrintBUTTON(buf);
+  return 1;
 }
 
 
 /* this is an interesting cheat -- we call DecodeEvent to print an event */
 /* should work, but its never been tried */
 int
-PrintEVENTFORM(unsigned char *buf)
+PrintEVENTFORM(const unsigned char *buf)
 {
   /* print an EVENT_FORM -- event format */
   DecodeEvent(-1, buf, (long)-1);
+  return 32;
 }
 
 /* ************************************************************ */
 
 int
 PrintENUMERATED(
-     unsigned char *buf,
+     const unsigned char *buf,
      short   length,
      struct ValueListEntry  *ValueList)
 {
@@ -624,16 +642,18 @@ PrintENUMERATED(
     p = p->Next;
 
   if (p != NULL)
-    return fprintf(stdout, "%s", p->Name);
+    fprintf(stdout, "%s", p->Name);
   else
-    return fprintf(stdout, "**INVALID** (%d)", n);
+    fprintf(stdout, "**INVALID** (%ld)", n);
+
+  return length;
 }
 
 /* ************************************************************ */
 
 int
 PrintSET(
-     unsigned char *buf,
+     const unsigned char *buf,
      short   length,
      struct ValueListEntry  *ValueList)
 {
@@ -674,9 +694,11 @@ PrintSET(
     }
 
   if (MatchesAll)
-    return fprintf(stdout, "<ALL>");
+    fprintf(stdout, "<ALL>");
   else if (!FoundOne)
-    return fprintf(stdout, "0");
+    fprintf(stdout, "0");
+
+  return length;
 }
 
 
@@ -686,13 +708,16 @@ PrintSET(
 /* ************************************************************ */
 
 void
-PrintField(buf, start, length, FieldType, name)
-     unsigned char *buf;
-     short   start;
-     short   length;
-     short   FieldType;
-     char   *name;
+PrintField (
+    const unsigned char *buf,
+    short   start,
+    short   length,
+    short   FieldType,
+    const char *name)
 {
+  if (Verbose == 0)
+    return;
+  
   if (length == 0)
     return;
 
@@ -736,11 +761,12 @@ PrintField(buf, start, length, FieldType, name)
 /* print a list of things.  The things are of type <ListType>.
    They start at <buf>.  There are <number> things in the list */
 
-long PrintList(buf, number, ListType, name)
-     unsigned char *buf;
-     long   number;
-     short   ListType;
-     char   *name;
+long
+PrintList (
+    const unsigned char *buf,
+    long   number,
+    short   ListType,
+    const char *name)
 {
   long    n;
   long    i;
@@ -749,7 +775,7 @@ long PrintList(buf, number, ListType, name)
   if (number == 0)
     return(0);
 
-  fprintf(stdout, "%s%20s: (%d)\n", Leader, name, number);
+  fprintf(stdout, "%s%20s: (%ld)\n", Leader, name, number);
   if (Verbose < 2)
     return(0);
 
@@ -787,10 +813,11 @@ long PrintList(buf, number, ListType, name)
 /* print a list of STRs.  Similar to PrintList
    They start at <buf>.  There are <number> things in the list */
 
-long PrintListSTR(buf, number, name)
-     unsigned char *buf;
-     long   number;
-     char   *name;
+long
+PrintListSTR (
+    const unsigned char *buf,
+    long   number,
+    const char *name)
 {
   long    n;
   long    i;
@@ -799,7 +826,7 @@ long PrintListSTR(buf, number, name)
   if (number == 0)
     return(0);
 
-  fprintf(stdout, "%s%20s: (%d)\n", Leader, name, number);
+  fprintf(stdout, "%s%20s: (%ld)\n", Leader, name, number);
   if (Verbose < 2)
     return(0);
 
@@ -827,9 +854,9 @@ long PrintListSTR(buf, number, name)
 
 int
 PrintBytes(
-    unsigned char   buf[],
+    const unsigned char *buf,
     long   number,
-    char   *name)
+    const char *name)
 {
   /* print a list of BYTE -- 8-bit character */
   long   i;
@@ -868,9 +895,9 @@ PrintBytes(
 
 int
 PrintString8(
-    unsigned char    buf[],
+    const unsigned char *buf,
     int   number,
-    char   *name)
+    const char *name)
 {
   short   i;
 
@@ -890,11 +917,11 @@ PrintString8(
 
 int
 PrintString16(
-    unsigned char    buf[],
+    const unsigned char *buf,
     int   number,
-    char   *name)
+    const char *name)
 {
-  short   i;
+  long   i;
   unsigned short   c;
 
   if (number == 0)
@@ -903,7 +930,7 @@ PrintString16(
   fprintf(stdout, "%s%20s: \"", Leader, name);
   for (i = 0; i < number; i += 2)
     {
-      c = IShort(&buf[i]);
+      c = IChar2B(&buf[i]);
       fprintf(stdout, "%s", printrep(c));
     }
   fprintf(stdout, "\"\n");
@@ -911,6 +938,54 @@ PrintString16(
   return(number);
 }
 
+void
+PrintTString8(
+    const unsigned char *buf,
+    long   number,
+    const char *name)
+{
+  long   i;
+  int	off;
+
+  if (number == 0)
+    return;
+
+  off = 0;
+  if (TranslateText)
+    off = 0x20;
+  fprintf(stdout, "%s%20s: \"", Leader, name);
+  for (i = 0; i < number; i++)
+    fprintf(stdout, "%s", printrep(buf[i] + off));
+  fprintf(stdout, "\"\n");
+}
+
+
+/* print a String of CHAR2B -- 16-bit characters */
+void
+PrintTString16(
+    const unsigned char *buf,
+    long   number,
+    const char *name)
+{
+  long   i;
+  unsigned short   c;
+  int	off;
+
+  if (number == 0)
+      return;
+
+  off = 0;
+  if (TranslateText)
+    off = 0x20;
+  fprintf(stdout, "%s%20s: \"", Leader, name);
+  for (i = 0; i < number; i += 2)
+    {
+      c = IChar2B(&buf[i]);
+      fprintf(stdout, "%s", printrep(c + off));
+    }
+  fprintf(stdout, "\"\n");
+}
+
 /* ************************************************************ */
 /*								*/
 /*								*/
@@ -926,11 +1001,11 @@ PrintString16(
 
 void
 PrintValues(    
-    unsigned char   *control,
+    const unsigned char   *control,
     int     clength,
     int     ctype,
-    unsigned char   *values,
-    char   *name)
+    const unsigned char   *values,
+    const char            *name)
 {
   long    cmask;
   struct ValueListEntry  *p;
@@ -955,7 +1030,11 @@ PrintValues(
     {
       if ((p->Value & cmask) != 0)
 	{
-	  short   m = 4 - p->Length;
+	  short m;
+	  if (littleEndian)
+	    m=0;
+	  else
+	    m = 4 - p->Length;
 	  PrintField(values, m, p->Length, p->Type, p->Name);
 	  values += 4;
 	}
@@ -972,11 +1051,11 @@ PrintValues(
 /* PolyText8 and PolyText16 take lists of characters with possible
    font changes in them. */
 
-int
+void
 PrintTextList8(
-    unsigned char *buf,
+    const unsigned char *buf,
     int     length,
-    char   *name)
+    const char *name)
 {
   short   n;
 
@@ -987,24 +1066,24 @@ PrintTextList8(
       if (n != 255)
 	{
 	  PrintField(buf, 1, 1, INT8, "delta");
-	  PrintString8(&buf[2], n, "text item 8 string");
+	  PrintTString8(&buf[2], (long)n, "text item 8 string");
 	  buf += n + 2;
 	  length -= n + 2;
 	}
       else
 	{
 	  PrintField(buf, 1, 4, FONT, "font-shift-id");
-	  buf += 5;
-	  length -= 5;
+	  buf += 4;
+	  length -= 4;
 	}
     }
 }
 
-int
+void
 PrintTextList16(
-    unsigned char *buf,
+    const unsigned char *buf,
     int     length,
-    char   *name)
+    const char *name)
 {
   short   n;
 
@@ -1015,15 +1094,15 @@ PrintTextList16(
       if (n != 255)
 	{
 	  PrintField(buf, 1, 1, INT8, "delta");
-	  PrintString16(&buf[2], n, "text item 16 string");
+	  PrintTString16(&buf[2], (long)n, "text item 16 string");
 	  buf += n + 2;
 	  length -= n + 2;
 	}
       else
 	{
 	  PrintField(buf, 1, 4, FONT, "font-shift-id");
-	  buf += 5;
-	  length -= 5;
+	  buf += 4;
+	  length -= 4;
 	}
     }
 }
@@ -1036,12 +1115,12 @@ PrintTextList16(
 
 #define MAXline 78
 
-static void
+void
 DumpHexBuffer(
-    unsigned char *buf,
+    const unsigned char *buf,
     long    n)
 {
-  short   i;
+  long   i;
   short   column;
   char    h[6] /* one hex or octal character */ ;
 
@@ -1062,3 +1141,41 @@ DumpHexBuffer(
       column += 3;
     }
 }
+
+void
+PrintValueRec (
+    unsigned long   key,
+    unsigned long   cmask,
+    short   ctype)
+{
+    unsigned char   *values;
+    struct ValueListEntry  *p;
+    ValuePtr	    value;
+
+    value = GetValueRec (key);
+    if (!value)
+	return;
+    values = (unsigned char *) value->values;
+    
+    /* now if it is zero, ignore and return */
+    if (cmask == 0)
+	return;
+
+    /* there are bits in the controlling bitmask, figure out which */
+    /* the ctype is a set type, so this code is similar to PrintSET */
+    ModifyIndentLevel(1);
+    for (p = TD[ctype].ValueList; p != NULL; p = p->Next)
+    {
+	if ((p->Value & cmask) != 0)
+	{
+	    short m;
+	    if (littleEndian)
+		m=0;
+	    else
+		m = 4 - p->Length;
+	    PrintField(values, m, p->Length, p->Type, p->Name);
+	}
+	values += 4;
+    }
+    ModifyIndentLevel(-1);
+}
